@@ -20,6 +20,7 @@
 #include "cluster_mgr.h"
 #include "globals.h"
 #include "comm_headers.h"
+#include "misc.h"
 
 #include "copyright.cf"
 
@@ -41,7 +42,7 @@ static int setup_local_socket(int backlog);
 static inline void process_signals(void);
 
 LOGSYS_DECLARE_SYSTEM (NULL,
-	LOG_MODE_OUTPUT_STDERR | LOG_MODE_OUTPUT_SYSLOG_THREADED | LOG_MODE_OUTPUT_FILE | LOG_MODE_DISPLAY_DEBUG | LOG_MODE_BUFFER_BEFORE_CONFIG,
+	LOG_MODE_OUTPUT_STDERR | LOG_MODE_OUTPUT_SYSLOG_THREADED | LOG_MODE_OUTPUT_FILE | LOG_MODE_BUFFER_BEFORE_CONFIG,
 	LOGDIR "/ccs.log",
 	SYSLOGFACILITY);
 
@@ -57,12 +58,15 @@ int main(int argc, char *argv[]){
   int addr_size=0;
   fd_set rset, tmp_set;
   char *msg;
+  unsigned int logmode;
 
   msg = parse_cli_args(argc, argv);
 
-  logsys_config_mode_set (LOG_MODE_OUTPUT_STDERR | LOG_MODE_OUTPUT_SYSLOG_THREADED | LOG_MODE_OUTPUT_FILE | LOG_MODE_DISPLAY_DEBUG | LOG_MODE_FLUSH_AFTER_CONFIG);
+  if(getenv("CCS_DEBUGLOG"))
+    debug = 1;
 
-  if(debug || getenv("CCS_DEBUGLOG"))
+  /* enable debug as early as possible */
+  if(debug)
     logsys_config_priority_set (LOG_LEVEL_DEBUG);
 
   if(check_cluster_conf()){
@@ -70,6 +74,15 @@ int main(int argc, char *argv[]){
     exit(EXIT_FAILURE);
   }
 
+  logmode = logsys_config_mode_get();
+
+  if(logmode & LOG_MODE_BUFFER_BEFORE_CONFIG) {
+    log_printf(LOG_DEBUG, "Using default CCS logsys config options\n");
+    logmode &= ~LOG_MODE_BUFFER_BEFORE_CONFIG;
+    logmode |= LOG_MODE_FLUSH_AFTER_CONFIG;
+    logsys_config_mode_set (logmode);
+  }
+
   daemonize();
 
   print_start_msg(msg);
@@ -528,6 +541,7 @@ static int check_cluster_conf(void){
 	      "valid copy from the network.\n", config_file_location);
       return -1;
     }
+    set_ccs_logging(doc);
     xmlFreeDoc(doc);
   } else {
     /* no cluster.conf file.  This is fine, just need to get it from the network */
@@ -537,6 +551,7 @@ static int check_cluster_conf(void){
     }
   }
 
+  CCSEXIT("check_cluster_conf");
   return 0;
 }
 
