@@ -1,16 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include <linux/sched.h>
 #include <linux/slab.h>
 #include <linux/smp_lock.h>
@@ -675,8 +662,7 @@ gfs_ea_get_copy(struct gfs_inode *ip,
 		       GFS_EA_DATA_LEN(el->el_ea));
 		return 0;
 	} else
-		return ea_get_unstuffed(ip, el->el_ea,
-					data);
+		return ea_get_unstuffed(ip, el->el_ea, data);
 }
 
 /**
