@@ -1,16 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 /*
  *  In-core (memory/RAM) structures.
  *  These do not appear on-disk.  See gfs_ondisk.h for on-disk structures.
@@ -227,6 +214,7 @@ struct gfs_rgrpd {
 	struct list_head rd_list;       /* On-disk-order list of all rgrps */
 	struct list_head rd_list_mru;   /* Most Recently Used list of all rgs */
 	struct list_head rd_recent;     /* recently used rgrps */
+	uint32_t rd_try_counter;        /* # of times we fail a try lock */
 
 	struct gfs_glock *rd_gl;        /* Glock for this rgrp */
 
@@ -511,7 +499,7 @@ struct gfs_glock {
 	struct gfs_holder *gl_req_gh; /* Holder for request being serviced */
 	gfs_glop_bh_t gl_req_bh;  /* The bottom half to execute */
 
-	lm_lock_t *gl_lock;       /* Lock module's private lock data */
+	void *gl_lock;            /* Lock module's private lock data */
 	char *gl_lvb;             /* Lock Value Block */
 	atomic_t gl_lvb_count;    /* LVB recursive usage (hold/unhold) count */
 
@@ -616,6 +604,7 @@ struct gfs_inode {
 
 	unsigned int i_greedy; /* The amount of time to be greedy */
 	unsigned long i_last_pfault; /* The time of the last page fault */
+	struct address_space_operations gfs_file_aops;
 };
 
 /*
@@ -816,12 +805,23 @@ struct gfs_trans {
 #define GFS_GLOCKD_DEFAULT (1)
 #define GFS_GLOCKD_MAX (32)
 
+#define GFS_QUOTA_DEFAULT      GFS_QUOTA_OFF
+#define GFS_QUOTA_OFF          0
+#define GFS_QUOTA_ACCOUNT      1
+#define GFS_QUOTA_ON           2
+
+#define GFS_DATA_DEFAULT       GFS_DATA_ORDERED
+#define GFS_DATA_WRITEBACK     1
+#define GFS_DATA_ORDERED       2
+
+
 struct gfs_args {
 	char ar_lockproto[GFS_LOCKNAME_LEN]; /* The name of the Lock Protocol */
 	char ar_locktable[GFS_LOCKNAME_LEN]; /* The name of the Lock Table */
 	char ar_hostdata[GFS_LOCKNAME_LEN]; /* The host specific data */
 
-        /*
+	int ar_spectator; /* Don't get a journal because we're always RO. */
+	/*
 	 * GFS can invoke some flock and disk caching optimizations if it is
 	 * not in a cluster, i.e. is a local filesystem.  The chosen lock
 	 * module tells GFS, at mount time, if it supports clustering.
@@ -842,6 +842,7 @@ struct gfs_args {
 				       (more daemons => faster cleanup)  */
 	int ar_posix_acls; /* Enable posix acls */
 	int ar_suiddir; /* suiddir support */
+	int ar_noquota; /* Turn off quota support */
 };
 
 struct gfs_tune {
@@ -864,6 +865,7 @@ struct gfs_tune {
 	unsigned int gt_logd_secs; /* Update log tail as AIL flushes */
 	unsigned int gt_quotad_secs; /* Sync changes to quota file, clean*/
 	unsigned int gt_inoded_secs; /* Toss unused inodes */
+	unsigned int gt_glock_purge; /* Purge glock */
 
 	unsigned int gt_quota_simul_sync; /* Max # quotavals to sync at once */
 	unsigned int gt_quota_warn_period; /* Secs between quota warn msgs */
@@ -888,6 +890,8 @@ struct gfs_tune {
 	unsigned int gt_greedy_default;
 	unsigned int gt_greedy_quantum;
 	unsigned int gt_greedy_max;
+	unsigned int gt_rgrp_try_threshold;
+	unsigned int gt_statfs_fast;
 };
 
 /*
@@ -915,14 +919,6 @@ struct gfs_gl_hash_bucket {
 #define SDF_JOURNAL_LIVE        (0)  /* Journaling is active (journal is writeable)*/
 #define SDF_SHUTDOWN            (1)  /* FS abnormaly shutdown */
 
-/* Run (1) / stop (0) flags for various daemons */
-#define SDF_SCAND_RUN           (2)  /* Put unused glocks on reclaim queue */
-#define SDF_GLOCKD_RUN          (3)  /* Reclaim (dealloc) unused glocks */
-#define SDF_RECOVERD_RUN        (4)  /* Recover journal of a crashed node */
-#define SDF_LOGD_RUN            (5)  /* Update log tail after AIL flushed */
-#define SDF_QUOTAD_RUN          (6)  /* Sync quota changes to file, cleanup */
-#define SDF_INODED_RUN          (7)  /* Deallocate unlinked inodes */
-
 /* (Re)mount options from Linux VFS */
 #define SDF_NOATIME             (8)  /* Don't change access time */
 #define SDF_ROFS                (9)  /* Read-only mode */
@@ -958,6 +954,13 @@ struct gfs_sbd {
 
 	struct gfs_tune sd_tune;	/* Filesystem tuning structure */
 
+	/* statfs */
+	struct inode *sd_statfs_inode;
+	spinlock_t sd_statfs_spin;
+	struct gfs_statfs_change_host sd_statfs_master;
+	struct gfs_statfs_change_host sd_statfs_local;
+	unsigned long sd_statfs_sync_time;
+
 	/* Resource group stuff */
 
 	struct gfs_inode *sd_riinode;	/* Resource Index (rindex) inode */
@@ -1047,6 +1050,7 @@ struct gfs_sbd {
 
 	/* Scan for glocks and inodes to toss from memory */
 	struct task_struct *sd_scand_process; /* Scand places on reclaim list*/
+	struct task_struct *sd_glockd_process[GFS_GLOCKD_MAX];
 	unsigned int sd_glockd_num;    /* # of glockd procs to do reclaiming*/
 
 	/* Recover journal of a crashed node */
@@ -1061,10 +1065,6 @@ struct gfs_sbd {
 	/* Clean up unused inode structures */
 	struct task_struct *sd_inoded_process;
 
-	/* Support for starting/stopping daemons */
-	struct semaphore sd_thread_lock;
-	struct completion sd_thread_completion;
-
 	/*  Log stuff  */
 
 	/* Transaction lock protects the following from one another:
@@ -1189,6 +1189,10 @@ struct gfs_sbd {
 	atomic_t sd_ops_vm;
 
 	char sd_fsname[256];
+	char sd_table_name[256];
+	char sd_proto_name[256];
+
+	struct kobject sd_kobj;
 
 	/* Debugging crud */
 
