@@ -2,7 +2,7 @@
 #include <linux/slab.h>
 #include <linux/smp_lock.h>
 #include <linux/spinlock.h>
-#include <linux/semaphore.h>
+#include <asm/semaphore.h>
 #include <linux/completion.h>
 #include <linux/buffer_head.h>
 #include <linux/posix_acl.h>
@@ -180,17 +180,10 @@ gfs_iget(struct gfs_inode *ip, int create)
 		memcpy(&ip->gfs_file_aops, &gfs_file_aops,
 			   sizeof(struct address_space_operations));
 		tmp->i_mapping->a_ops = &ip->gfs_file_aops;
-		if (sdp->sd_args.ar_localflocks) {
-			if (gfs_is_jdata(ip))
-				tmp->i_fop = &gfs_file_fops_nolock_jdata;
-			else
-				tmp->i_fop = &gfs_file_fops_nolock;
-		} else {
-			if (gfs_is_jdata(ip))
-				tmp->i_fop = &gfs_file_fops_jdata;
-			else
-				tmp->i_fop = &gfs_file_fops;
-		}
+		if (sdp->sd_args.ar_localflocks)
+			tmp->i_fop = &gfs_file_fops_nolock;
+		else
+			tmp->i_fop = &gfs_file_fops;
 	} else if (ip->i_di.di_type == GFS_FILE_DIR) {
 		tmp->i_op = &gfs_dir_iops;
 		if (sdp->sd_args.ar_localflocks)
@@ -341,10 +334,6 @@ inode_create(struct gfs_glock *i_gl, struct gfs_inum *inum,
 	gfs_glock_hold(i_gl);
 	set_gl2ip(i_gl, ip);
 
-	/* initialize stat counter and timestamp */
-	ip->i_dir_stats = 0;
-	do_gettimeofday(&ip->i_dir_stat_st);
-
 	atomic_inc(&sdp->sd_inode_count);
 
 	*ipp = ip;
@@ -921,7 +910,7 @@ gfs_lookupi(struct gfs_holder *d_gh, struct qstr *name,
 		return error;
 
 	if (!is_root) {
-		error = inode_permission(dip->i_vnode, MAY_EXEC);
+		error = permission(dip->i_vnode, MAY_EXEC, NULL);
 		if (error) {
 			gfs_glock_dq(d_gh);
 			return error;
@@ -963,7 +952,7 @@ gfs_lookupi(struct gfs_holder *d_gh, struct qstr *name,
 		}
 
 		if (!is_root) {
-			error = inode_permission(dip->i_vnode, MAY_EXEC);
+			error = permission(dip->i_vnode, MAY_EXEC, NULL);
 			if (error) {
 				gfs_glock_dq(d_gh);
 				gfs_glock_dq_uninit(i_gh);
@@ -1028,7 +1017,7 @@ create_ok(struct gfs_inode *dip, struct qstr *name, unsigned int type)
 {
 	int error;
 
-	error = inode_permission(dip->i_vnode, MAY_WRITE | MAY_EXEC);
+	error = permission(dip->i_vnode, MAY_WRITE | MAY_EXEC, NULL);
 	if (error)
 		return error;
 
@@ -1588,7 +1577,7 @@ gfs_unlink_ok(struct gfs_inode *dip, struct qstr *name, struct gfs_inode *ip)
 	if (IS_APPEND(dip->i_vnode))
 		return -EPERM;
 
-	error = inode_permission(dip->i_vnode, MAY_WRITE | MAY_EXEC);
+	error = permission(dip->i_vnode, MAY_WRITE | MAY_EXEC, NULL);
 	if (error)
 		return error;
 
