@@ -1,15 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -323,7 +311,7 @@ void parse_cmdline(int argc, char **argv)
       continue;
       
     case 'V':
-      printf("%s %s (built %s %s)\n", program_name, GNBD_RELEASE_NAME,
+      printf("%s %s (built %s %s)\n", program_name, RELEASE_VERSION,
              __DATE__, __TIME__);
       printf("%s\n", REDHAT_COPYRIGHT);
       exit(0);
