@@ -1,25 +1,5 @@
 #!/bin/bash
 
-#
-#  Copyright Red Hat, Inc. 2002-2004
-#  Copyright Mission Critical Linux, Inc. 2000
-#
-#  This program is free software; you can redistribute it and/or modify it
-#  under the terms of the GNU General Public License as published by the
-#  Free Software Foundation; either version 2, or (at your option) any
-#  later version.
-#
-#  This program is distributed in the hope that it will be useful, but
-#  WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; see the file COPYING.  If not, write to the
-#  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-#  MA 02139, USA.
-#
-
 #
 # File system (normal) mount/umount/fsck/etc. agent
 #
@@ -413,7 +393,7 @@ isMounted () {
 		ocf_log err "isMounted: Could not match $1 with a real device"
 		return $FAIL
 	fi
-	mp=$2
+	mp=$(readlink -f $2)
 	
 	while read tmp_dev tmp_mp
 	do
