@@ -522,13 +522,22 @@ network_link_up()
 	#
 	if [ "${intf_arg/bond/}" != "$intf_arg" ]; then
 		
+		#
+		# With Xen we must check for slaves of pbondX, not bondX
+		#
+		masterif=$intf_arg
+		line=$(ip link list | awk {'print $2'} | grep "p$intf_arg:")
+		if [ $? -eq 0 ] ; then
+			masterif="p$intf_arg"
+		fi
+		
 		#
 		# Bonded driver.  Check link of all slaves for this interface.
 		# If any link is up, the bonding driver is expected to route
 		# traffic through that link.  Thus, the entire bonded link
 		# is declared up.
 		#
-		slaves=$(findSlaves $intf_arg)
+		slaves=$(findSlaves $masterif)
 		if [ $? -ne 0 ]; then
 			ocf_log err "Error finding slaves of $intf_arg"
 			return 1
